package entity

type Sensor struct {
	SensorName string  `json:"sensor_name"` // 传感器名称
	SensorNear float64 `json:"sensor_near"` // 盲区距离
	SensorFar  float64 `json:"sensor_far"`  // 探测距离
	SensorX    float64 `json:"sensor_x"`    // 传感器横向偏移量（x轴）
	SensorY    float64 `json:"sensor_y"`    // 传感器纵向偏移量（y轴）
	SensorZ    float64 `json:"sensor_z"`    // 传感器安装高度（z轴）
	SensorH    float64 `json:"sensor_h"`    // 传感器横摆角
	SensorP    float64 `json:"sensor_p"`    // 传感器俯仰角
	SensorR    float64 `json:"sensor_r"`    // 传感器横滚角
}

// SensorParam 从数据闭环平台接收到的参数，通用
type SensorParam struct {
	Sensor
	SensorForH       float64 `json:"sensor_fovH"`        // 水平视场角
	SensorForV       float64 `json:"sensor_fovV"`        // 垂直视场角
	SensorForHOffset float64 `json:"sensor_fovH_offset"` // 水平视场角偏移量
	SensorForVOffset float64 `json:"sensor_fovV_offset"` // 垂直视场角偏移量
}

// SensorCamera 摄像头
type SensorCamera struct {
	Sensor
	SensorForH       float64 `json:"sensor_fovH"`       // 水平现场角
	SensorForV       float64 `json:"sensor_fovV"`       // 垂直现场角
	SensorResolution string  `json:"sensor_resolution"` // 分辨率
	SensorFrameRate  float64 `json:"sensor_frameRate"`  // 帧率
}

// SensorOgt 完美传感器
type SensorOgt struct {
	Sensor
	SensorForHLeft   float64 `json:"sensor_fovHLeft"`   // 水平现场角左
	SensorForHRight  float64 `json:"sensor_fovHRight"`  // 水平现场角右
	SensorForHTop    float64 `json:"sensor_fovVTop"`    // 垂直现场角顶
	SensorForHBottom float64 `json:"sensor_fovVBottom"` // 垂直现场角底
	SensorFilter     string  `json:"sensor_filter"`     // 目标物筛选序列（0,1,2）
	SensorDisplay    bool    `json:"sensor_display"`    // 显示目标物
	SensorMaxObjects int     `json:"sensor_maxObjects"` // 最大目标物个数
	SensorPort       int     `json:"sensor_port"`       // 端口
}
